@@ -3,7 +3,7 @@ package compose
 import (
 	"errors"
 
-	"git.sr.ht/~sircmpwn/aerc/widgets"
+	"git.sr.ht/~rjarry/aerc/widgets"
 )
 
 type Edit struct{}
@@ -12,19 +12,19 @@ func init() {
 	register(Edit{})
 }
 
-func (_ Edit) Aliases() []string {
+func (Edit) Aliases() []string {
 	return []string{"edit"}
 }
 
-func (_ Edit) Complete(aerc *widgets.Aerc, args []string) []string {
+func (Edit) Complete(aerc *widgets.Aerc, args []string) []string {
 	return nil
 }
 
-func (_ Edit) Execute(aerc *widgets.Aerc, args []string) error {
+func (Edit) Execute(aerc *widgets.Aerc, args []string) error {
 	if len(args) != 1 {
 		return errors.New("Usage: edit")
 	}
-	composer, _ := aerc.SelectedTab().(*widgets.Composer)
+	composer, _ := aerc.SelectedTabContent().(*widgets.Composer)
 	composer.ShowTerminal()
 	composer.FocusTerminal()
 	return nil
