@@ -7,11 +7,10 @@ import (
 	"os/exec"
 	"regexp"
 	"sort"
-	"strconv"
 	"time"
 
 	"git.sr.ht/~rjarry/aerc/commands"
-	"git.sr.ht/~rjarry/aerc/logging"
+	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/widgets"
 	mboxer "git.sr.ht/~rjarry/aerc/worker/mbox"
 	"git.sr.ht/~rjarry/aerc/worker/types"
@@ -93,12 +92,12 @@ func (Pipe) Execute(aerc *widgets.Aerc, args []string) error {
 			return
 		}
 		go func() {
-			defer logging.PanicHandler()
+			defer log.PanicHandler()
 
 			defer pipe.Close()
 			_, err := io.Copy(pipe, reader)
 			if err != nil {
-				logging.Errorf("failed to send data to pipe: %v", err)
+				log.Errorf("failed to send data to pipe: %v", err)
 			}
 		}()
 		err = ecmd.Run()
@@ -124,6 +123,18 @@ func (Pipe) Execute(aerc *widgets.Aerc, args []string) error {
 		h := newHelper(aerc)
 		store, err := h.store()
 		if err != nil {
+			if mv, ok := provider.(*widgets.MessageViewer); ok {
+				mv.MessageView().FetchFull(func(reader io.Reader) {
+					if background {
+						doExec(reader)
+					} else {
+						doTerm(reader,
+							fmt.Sprintf("%s <%s",
+								cmd[0], title))
+					}
+				})
+				return nil
+			}
 			return err
 		}
 		uids, err = h.markedOrSelectedUids()
@@ -155,7 +166,7 @@ func (Pipe) Execute(aerc *widgets.Aerc, args []string) error {
 		})
 
 		go func() {
-			defer logging.PanicHandler()
+			defer log.PanicHandler()
 
 			select {
 			case <-done:
@@ -169,11 +180,11 @@ func (Pipe) Execute(aerc *widgets.Aerc, args []string) error {
 				}
 			}
 
-			is_git_patches := true
+			is_git_patches := false
 			for _, msg := range messages {
 				info := store.Messages[msg.Content.Uid]
-				if info == nil || !gitMessageIdRe.MatchString(info.Envelope.MessageId) {
-					is_git_patches = false
+				if info != nil && patchSeriesRe.MatchString(info.Envelope.Subject) {
+					is_git_patches = true
 					break
 				}
 			}
@@ -186,9 +197,7 @@ func (Pipe) Execute(aerc *widgets.Aerc, args []string) error {
 					if infoi == nil || infoj == nil {
 						return false
 					}
-					msgidi := padGitMessageId(infoi.Envelope.MessageId)
-					msgidj := padGitMessageId(infoj.Envelope.MessageId)
-					return msgidi < msgidj
+					return infoi.Envelope.Subject < infoj.Envelope.Subject
 				})
 			}
 
@@ -218,13 +227,16 @@ func (Pipe) Execute(aerc *widgets.Aerc, args []string) error {
 			}
 		})
 	}
-	provider.Store().Marker().ClearVisualMark()
+	if store := provider.Store(); store != nil {
+		store.Marker().ClearVisualMark()
+	}
 	return nil
 }
 
 func newMessagesReader(messages []*types.FullMessage, useMbox bool) io.Reader {
 	pr, pw := io.Pipe()
 	go func() {
+		defer log.PanicHandler()
 		defer pw.Close()
 		for _, msg := range messages {
 			var err error
@@ -234,28 +246,13 @@ func newMessagesReader(messages []*types.FullMessage, useMbox bool) io.Reader {
 				_, err = io.Copy(pw, msg.Content.Reader)
 			}
 			if err != nil {
-				logging.Warnf("failed to write data: %v", err)
+				log.Warnf("failed to write data: %v", err)
 			}
 		}
 	}()
 	return pr
 }
 
-var gitMessageIdRe = regexp.MustCompile(`^(\d+\.\d+)-(\d+)-(.+)$`)
-
-// Git send-email Message-Id headers have the following format:
-//
-//	DATETIME.PID-NUM-COMMITTER
-//
-// Return a copy of the message id with NUM zero-padded to three characters.
-func padGitMessageId(msgId string) string {
-	matches := gitMessageIdRe.FindStringSubmatch(msgId)
-	if matches == nil {
-		return msgId
-	}
-	number, err := strconv.Atoi(matches[2])
-	if err != nil {
-		return msgId
-	}
-	return fmt.Sprintf("%s-%03d-%s", matches[1], number, matches[3])
-}
+var patchSeriesRe = regexp.MustCompile(
+	`^.*\[(RFC )?PATCH( [^\]]+)? \d+/\d+] .+$`,
+)
