@@ -3,7 +3,8 @@ package msg
 import (
 	"errors"
 
-	"git.sr.ht/~rjarry/aerc/lib/statusline"
+	"git.sr.ht/~rjarry/aerc/lib/state"
+	"git.sr.ht/~rjarry/aerc/lib/ui"
 	"git.sr.ht/~rjarry/aerc/widgets"
 )
 
@@ -34,9 +35,8 @@ func (ToggleThreads) Execute(aerc *widgets.Aerc, args []string) error {
 	if err != nil {
 		return err
 	}
-	defer store.Reselect(store.Selected())
-	store.SetBuildThreads(!store.BuildThreads())
-	acct.SetStatus(statusline.Threading(store.BuildThreads()))
-	acct.Messages().Invalidate()
+	store.SetThreadedView(!store.ThreadedView())
+	acct.SetStatus(state.Threading(store.ThreadedView()))
+	ui.Invalidate()
 	return nil
 }
