@@ -9,9 +9,7 @@ import (
 	"time"
 
 	"git.sr.ht/~rjarry/aerc/lib"
-	"git.sr.ht/~rjarry/aerc/lib/ui"
-	"git.sr.ht/~rjarry/aerc/logging"
-	"git.sr.ht/~rjarry/aerc/models"
+	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/widgets"
 	"github.com/emersion/go-message/mail"
 )
@@ -56,10 +54,10 @@ func (Unsubscribe) Execute(aerc *widgets.Aerc, args []string) error {
 	if len(methods) == 0 {
 		return fmt.Errorf("no methods found to unsubscribe")
 	}
-	logging.Infof("unsubscribe: found %d methods", len(methods))
+	log.Debugf("unsubscribe: found %d methods", len(methods))
 
 	unsubscribe := func(method *url.URL) {
-		logging.Infof("unsubscribe: trying to unsubscribe using %s", method.Scheme)
+		log.Debugf("unsubscribe: trying to unsubscribe using %s", method.Scheme)
 		var err error
 		switch strings.ToLower(method.Scheme) {
 		case "mailto":
@@ -151,26 +149,17 @@ func unsubscribeMailto(aerc *widgets.Aerc, u *url.URL) error {
 	composer, err := widgets.NewComposer(
 		aerc,
 		acct,
-		aerc.Config(),
 		acct.AccountConfig(),
 		acct.Worker(),
 		"",
 		h,
-		models.OriginalMail{},
+		nil,
 	)
 	if err != nil {
 		return err
 	}
 	composer.SetContents(strings.NewReader(u.Query().Get("body")))
-	tab := aerc.NewTab(composer, "unsubscribe")
-	composer.OnHeaderChange("Subject", func(subject string) {
-		if subject == "" {
-			tab.Name = "unsubscribe"
-		} else {
-			tab.Name = subject
-		}
-		ui.Invalidate()
-	})
+	composer.Tab = aerc.NewTab(composer, "unsubscribe")
 	composer.FocusTerminal()
 	return nil
 }
@@ -185,7 +174,9 @@ func unsubscribeHTTP(aerc *widgets.Aerc, u *url.URL) error {
 			switch option {
 			case "Yes":
 				go func() {
-					if err := lib.XDGOpen(u.String()); err != nil {
+					defer log.PanicHandler()
+					mime := fmt.Sprintf("x-scheme-handler/%s", u.Scheme)
+					if err := lib.XDGOpenMime(u.String(), mime, nil); err != nil {
 						aerc.PushError("Unsubscribe:" + err.Error())
 					}
 				}()
