@@ -5,10 +5,11 @@ import (
 	"sync"
 	"time"
 
+	"git.sr.ht/~rjarry/aerc/lib/iterator"
 	"git.sr.ht/~rjarry/aerc/lib/marker"
 	"git.sr.ht/~rjarry/aerc/lib/sort"
 	"git.sr.ht/~rjarry/aerc/lib/ui"
-	"git.sr.ht/~rjarry/aerc/logging"
+	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/types"
 )
@@ -17,7 +18,6 @@ import (
 type MessageStore struct {
 	sync.Mutex
 	Deleted  map[uint32]interface{}
-	DirInfo  models.DirectoryInfo
 	Messages map[uint32]*models.MessageInfo
 	Sorting  bool
 
@@ -37,10 +37,13 @@ type MessageStore struct {
 	filter      []string
 
 	sortCriteria []*types.SortCriterion
+	sortDefault  []*types.SortCriterion
 
-	threadedView bool
-	buildThreads bool
-	builder      *ThreadBuilder
+	threadedView       bool
+	reverseThreadOrder bool
+	sortThreadSiblings bool
+	buildThreads       bool
+	builder            *ThreadBuilder
 
 	// Map of uids we've asked the worker to fetch
 	onUpdate       func(store *MessageStore) // TODO: multiple onUpdate handlers
@@ -63,35 +66,40 @@ type MessageStore struct {
 
 	// threads mutex protects the store.threads and store.threadCallback
 	threadsMutex sync.Mutex
+
+	iterFactory iterator.Factory
+	onSelect    func(*models.MessageInfo)
 }
 
 const MagicUid = 0xFFFFFFFF
 
 func NewMessageStore(worker *types.Worker,
-	dirInfo *models.DirectoryInfo,
 	defaultSortCriteria []*types.SortCriterion,
 	thread bool, clientThreads bool, clientThreadsDelay time.Duration,
+	reverseOrder bool, reverseThreadOrder bool, sortThreadSiblings bool,
 	triggerNewEmail func(*models.MessageInfo),
-	triggerDirectoryChange func(),
+	triggerDirectoryChange func(), onSelect func(*models.MessageInfo),
 ) *MessageStore {
-	if !dirInfo.Caps.Thread {
+	if !worker.Backend.Capabilities().Thread {
 		clientThreads = true
 	}
 
 	return &MessageStore{
 		Deleted:  make(map[uint32]interface{}),
-		DirInfo:  *dirInfo,
 		Messages: make(map[uint32]*models.MessageInfo),
 
 		selectedUid: MagicUid,
 
 		bodyCallbacks: make(map[uint32][]func(*types.FullMessage)),
 
-		threadedView: thread,
-		buildThreads: clientThreads,
+		threadedView:       thread,
+		buildThreads:       clientThreads,
+		reverseThreadOrder: reverseThreadOrder,
+		sortThreadSiblings: sortThreadSiblings,
 
 		filter:       []string{"filter"},
 		sortCriteria: defaultSortCriteria,
+		sortDefault:  defaultSortCriteria,
 
 		pendingBodies:  make(map[uint32]interface{}),
 		pendingHeaders: make(map[uint32]interface{}),
@@ -104,6 +112,9 @@ func NewMessageStore(worker *types.Worker,
 		triggerDirectoryChange: triggerDirectoryChange,
 
 		threadBuilderDelay: clientThreadsDelay,
+
+		iterFactory: iterator.NewFactory(reverseOrder),
+		onSelect:    onSelect,
 	}
 }
 
@@ -200,11 +211,8 @@ func (store *MessageStore) Update(msg types.WorkerMessage) {
 	updateThreads := false
 	directoryChange := false
 	switch msg := msg.(type) {
-	case *types.DirectoryInfo:
-		store.DirInfo = *msg.Info
-		if !msg.SkipSort {
-			store.Sort(store.sortCriteria, nil)
-		}
+	case *types.OpenDirectory:
+		store.Sort(store.sortCriteria, nil)
 		update = true
 	case *types.DirectoryContents:
 		newMap := make(map[uint32]*models.MessageInfo)
@@ -222,31 +230,32 @@ func (store *MessageStore) Update(msg types.WorkerMessage) {
 			store.runThreadBuilderNow()
 		}
 	case *types.DirectoryThreaded:
-		var uids []uint32
 		newMap := make(map[uint32]*models.MessageInfo)
 
-		for i := len(msg.Threads) - 1; i >= 0; i-- {
-			_ = msg.Threads[i].Walk(func(t *types.Thread, level int, currentErr error) error {
-				uid := t.Uid
-				uids = append([]uint32{uid}, uids...)
-				if msg, ok := store.Messages[uid]; ok {
-					newMap[uid] = msg
-				} else {
-					newMap[uid] = nil
-					directoryChange = true
-				}
-				return nil
-			})
-		}
-		store.Messages = newMap
-		store.uids = uids
+		store.builder = NewThreadBuilder(store.iterFactory)
+		store.builder.RebuildUids(msg.Threads, store.reverseThreadOrder)
+		store.uids = store.builder.Uids()
 		store.threads = msg.Threads
+
+		for _, uid := range store.uids {
+			if msg, ok := store.Messages[uid]; ok {
+				newMap[uid] = msg
+			} else {
+				newMap[uid] = nil
+				directoryChange = true
+			}
+		}
+
+		store.Messages = newMap
 		update = true
 	case *types.MessageInfo:
 		if existing, ok := store.Messages[msg.Info.Uid]; ok && existing != nil {
 			merge(existing, msg.Info)
 		} else if msg.Info.Envelope != nil {
 			store.Messages[msg.Info.Uid] = msg.Info
+			if store.selectedUid == msg.Info.Uid {
+				store.onSelect(msg.Info)
+			}
 		}
 		if msg.NeedsFlags {
 			store.Lock()
@@ -254,16 +263,9 @@ func (store *MessageStore) Update(msg types.WorkerMessage) {
 			store.Unlock()
 			store.fetchFlags()
 		}
-		seen := false
-		recent := false
-		for _, flag := range msg.Info.Flags {
-			if flag == models.RecentFlag {
-				recent = true
-			} else if flag == models.SeenFlag {
-				seen = true
-			}
-		}
-		if !seen && recent {
+		seen := msg.Info.Flags.Has(models.SeenFlag)
+		recent := msg.Info.Flags.Has(models.RecentFlag)
+		if !seen && recent && msg.Info.Envelope != nil {
 			store.triggerNewEmail(msg.Info)
 		}
 		if _, ok := store.pendingHeaders[msg.Info.Uid]; msg.Info.Envelope != nil && ok {
@@ -305,6 +307,9 @@ func (store *MessageStore) Update(msg types.WorkerMessage) {
 			}
 		}
 		store.uids = uids
+		if len(uids) == 0 {
+			store.Select(MagicUid)
+		}
 
 		var newResults []uint32
 		for _, res := range store.results {
@@ -355,11 +360,27 @@ func (store *MessageStore) update(threads bool) {
 	if store.onUpdateDirs != nil {
 		store.onUpdateDirs()
 	}
-	if store.BuildThreads() && store.ThreadedView() && threads {
-		store.runThreadBuilder()
+	if store.ThreadedView() && threads {
+		switch {
+		case store.BuildThreads():
+			store.runThreadBuilder()
+		default:
+			if store.builder == nil {
+				store.builder = NewThreadBuilder(store.iterFactory)
+			}
+			store.builder.RebuildUids(store.Threads(), store.reverseThreadOrder)
+		}
 	}
 }
 
+func (store *MessageStore) SetReverseThreadOrder(reverse bool) {
+	store.reverseThreadOrder = reverse
+}
+
+func (store *MessageStore) ReverseThreadOrder() bool {
+	return store.reverseThreadOrder
+}
+
 func (store *MessageStore) SetThreadedView(thread bool) {
 	store.threadedView = thread
 	if store.buildThreads {
@@ -379,6 +400,12 @@ func (store *MessageStore) Threads() []*types.Thread {
 	return store.threads
 }
 
+func (store *MessageStore) ThreadsIterator() iterator.Iterator {
+	store.threadsMutex.Lock()
+	defer store.threadsMutex.Unlock()
+	return store.iterFactory.NewIterator(store.threads)
+}
+
 func (store *MessageStore) ThreadedView() bool {
 	return store.threadedView
 }
@@ -388,9 +415,15 @@ func (store *MessageStore) BuildThreads() bool {
 }
 
 func (store *MessageStore) runThreadBuilder() {
+	if store.builder == nil {
+		store.builder = NewThreadBuilder(store.iterFactory)
+		for _, msg := range store.Messages {
+			store.builder.Update(msg)
+		}
+	}
 	if store.threadBuilderDebounce != nil {
 		if store.threadBuilderDebounce.Stop() {
-			logging.Infof("thread builder debounced")
+			log.Tracef("thread builder debounced")
 		}
 	}
 	store.threadBuilderDebounce = time.AfterFunc(store.threadBuilderDelay, func() {
@@ -402,13 +435,14 @@ func (store *MessageStore) runThreadBuilder() {
 // runThreadBuilderNow runs the threadbuilder without any debounce logic
 func (store *MessageStore) runThreadBuilderNow() {
 	if store.builder == nil {
-		store.builder = NewThreadBuilder()
+		store.builder = NewThreadBuilder(store.iterFactory)
 		for _, msg := range store.Messages {
 			store.builder.Update(msg)
 		}
 	}
 	// build new threads
-	th := store.builder.Threads(store.uids)
+	th := store.builder.Threads(store.uids, store.reverseThreadOrder,
+		store.sortThreadSiblings)
 
 	// save local threads to the message store variable and
 	// run callback if defined (callback should reposition cursor)
@@ -438,7 +472,7 @@ func (store *MessageStore) SelectedThread() *types.Thread {
 			return nil
 		})
 		if err != nil {
-			logging.Errorf("SelectedThread failed: %v", err)
+			log.Errorf("SelectedThread failed: %v", err)
 		}
 		if found {
 			break
@@ -516,12 +550,12 @@ func (store *MessageStore) Move(uids []uint32, dest string, createDest bool,
 	})
 }
 
-func (store *MessageStore) Flag(uids []uint32, flag models.Flag,
+func (store *MessageStore) Flag(uids []uint32, flags models.Flags,
 	enable bool, cb func(msg types.WorkerMessage),
 ) {
 	store.worker.PostAction(&types.FlagMessages{
 		Enable: enable,
-		Flag:   flag,
+		Flags:  flags,
 		Uids:   uids,
 	}, cb)
 }
@@ -544,14 +578,18 @@ func (store *MessageStore) Uids() []uint32 {
 	return store.uids
 }
 
+func (store *MessageStore) UidsIterator() iterator.Iterator {
+	return store.iterFactory.NewIterator(store.Uids())
+}
+
 func (store *MessageStore) Selected() *models.MessageInfo {
 	return store.Messages[store.selectedUid]
 }
 
 func (store *MessageStore) SelectedUid() uint32 {
 	if store.selectedUid == MagicUid && len(store.Uids()) > 0 {
-		uids := store.Uids()
-		store.selectedUid = uids[len(uids)-1]
+		iter := store.UidsIterator()
+		store.Select(store.Uids()[iter.StartIndex()])
 	}
 	return store.selectedUid
 }
@@ -562,10 +600,17 @@ func (store *MessageStore) Select(uid uint32) {
 		store.threadCallback = nil
 	}
 	store.threadsMutex.Unlock()
+	store.selectPriv(uid)
+}
+
+func (store *MessageStore) selectPriv(uid uint32) {
 	store.selectedUid = uid
 	if store.marker != nil {
 		store.marker.UpdateVisualMark()
 	}
+	if store.onSelect != nil {
+		store.onSelect(store.Selected())
+	}
 }
 
 func (store *MessageStore) NextPrev(delta int) {
@@ -574,28 +619,26 @@ func (store *MessageStore) NextPrev(delta int) {
 		return
 	}
 
-	uid := store.SelectedUid()
+	iter := store.iterFactory.NewIterator(uids)
 
-	newIdx := store.FindIndexByUid(uid)
+	newIdx := store.FindIndexByUid(store.SelectedUid())
 	if newIdx < 0 {
-		store.Select(uids[len(uids)-1])
+		store.Select(uids[iter.StartIndex()])
 		return
 	}
-
-	newIdx -= delta
-	if newIdx >= len(uids) {
-		newIdx = len(uids) - 1
-	} else if newIdx < 0 {
-		newIdx = 0
-	}
-
+	newIdx = iterator.MoveIndex(
+		newIdx,
+		delta,
+		iter,
+		iterator.FixBounds,
+	)
 	store.Select(uids[newIdx])
 
 	if store.BuildThreads() && store.ThreadedView() {
 		store.threadsMutex.Lock()
 		store.threadCallback = func() {
 			if uids := store.Uids(); len(uids) > newIdx {
-				store.selectedUid = uids[newIdx]
+				store.selectPriv(uids[newIdx])
 			}
 		}
 		store.threadsMutex.Unlock()
@@ -604,15 +647,7 @@ func (store *MessageStore) NextPrev(delta int) {
 	if store.marker != nil {
 		store.marker.UpdateVisualMark()
 	}
-
-	nextResultIndex := len(store.results) - store.resultIndex - 2*delta
-	if nextResultIndex < 0 || nextResultIndex >= len(store.results) {
-		return
-	}
-	nextResultUid := store.results[nextResultIndex]
-	if nextResultUid == store.SelectedUid() {
-		store.resultIndex += delta
-	}
+	store.updateResults()
 }
 
 func (store *MessageStore) Next() {
@@ -650,6 +685,16 @@ func (store *MessageStore) ApplySearch(results []uint32) {
 	store.NextResult()
 }
 
+// IsResult returns true if uid is a search result
+func (store *MessageStore) IsResult(uid uint32) bool {
+	for _, hit := range store.results {
+		if hit == uid {
+			return true
+		}
+	}
+	return false
+}
+
 func (store *MessageStore) SetFilter(args []string) {
 	store.filter = append(store.filter, args...)
 }
@@ -660,21 +705,38 @@ func (store *MessageStore) ApplyClear() {
 	if store.onFilterChange != nil {
 		store.onFilterChange(store)
 	}
-	store.Sort(nil, nil)
+	store.Sort(store.sortDefault, nil)
+}
+
+func (store *MessageStore) updateResults() {
+	if len(store.results) == 0 || store.resultIndex < 0 {
+		return
+	}
+	uid := store.SelectedUid()
+	for i, u := range store.results {
+		if uid == u {
+			store.resultIndex = i
+			break
+		}
+	}
 }
 
 func (store *MessageStore) nextPrevResult(delta int) {
 	if len(store.results) == 0 {
 		return
 	}
-	store.resultIndex += delta
-	if store.resultIndex >= len(store.results) {
-		store.resultIndex = 0
-	}
+	iter := store.iterFactory.NewIterator(store.results)
 	if store.resultIndex < 0 {
-		store.resultIndex = len(store.results) - 1
+		store.resultIndex = iter.StartIndex()
+	} else {
+		store.resultIndex = iterator.MoveIndex(
+			store.resultIndex,
+			delta,
+			iter,
+			iterator.WrapBounds,
+		)
 	}
-	store.Select(store.results[len(store.results)-store.resultIndex-1])
+	store.Select(store.results[store.resultIndex])
 	store.update(false)
 }
 
@@ -748,7 +810,7 @@ func (store *MessageStore) FindIndexByUid(uid uint32) int {
 
 // Capabilities returns a models.Capabilities struct or nil if not available
 func (store *MessageStore) Capabilities() *models.Capabilities {
-	return store.DirInfo.Caps
+	return store.worker.Backend.Capabilities()
 }
 
 // SelectedIndex returns the index of the selected message in the uid list or
