@@ -32,6 +32,9 @@ struct
        k v
      | #ceff as e ->
        `Let ((local x ty, e), k (local x ty))
+     | `CAMLparam (xs, c) ->
+	let Ty t = Type_C.ccomp c in
+	`CAMLparam (xs, (c, t) >>= k)
      | `LetConst (y, i, c) ->
        (* let x = (let const y = i in c) in e
           ~>
@@ -192,7 +195,8 @@ struct
                 release_runtime_system >>
                 `Let ((local x t, e), 
                       acquire_runtime_system >>
-                     (inj t (local x t) :> ccomp))
+                     (((inj t (local x t) :> ccomp), value) >>= fun x ->
+		      `CAMLreturnT (Ty value, x) :> ccomp))
              | `Return_errno, `Sequential -> 
                (`LetAssign (errno,
                            `Int Signed.SInt.zero,
@@ -206,7 +210,8 @@ struct
                            `Let ((local x t, e),
                                  (acquire_runtime_system >>
                                  (inj t (local x t) :> ccomp), value) >>= fun v ->
-                                 (pair_with_errno v :> ccomp))) : ccomp)
+                                 ((pair_with_errno v :> ccomp), value) >>= fun x ->
+				 `CAMLreturnT (Ty value, x))) : ccomp)
            end
          | Function (x, f, t) ->
            begin match prj f (local x value) with
@@ -217,9 +222,12 @@ struct
            end
       in
       let f' = name_params f in
-      `Function (`Fundec (stub_name, value_params f', Ty value),
-                 body [] f',
-                `Extern)
+      let vp = value_params f' in
+      `Function (`Fundec (stub_name, vp, Ty value),
+		 (match concurrency with
+		   `Unlocked -> `CAMLparam(List.map fst vp, body [] f')
+		 | `Sequential -> body [] f'),
+                 `Extern)
 
   let byte_fn : type a. string -> a Ctypes_static.fn -> int -> cfundef =
     fun fname fn nargs ->
@@ -362,10 +370,10 @@ struct
   let structure (type r) ~errno ~stub_name fmt fn args (result : r typ) =
     let open Ctypes in
     let s = structure_type stub_name in
-    let (_ : (_,_) field) = field s "job" lwt_unix_job in
+    let _ : (_,_) field = field s "job" lwt_unix_job in
     let () = match result with
-        Void -> let (_ : (_,_) field) = field s "result" int in ()
-      | result -> let (_ : (_,_) field) = field s "result" result in ()
+        Void -> let _ : (_,_) field = field s "result" int in ()
+      | result -> let _ : (_,_) field = field s "result" result in ()
     in
     let () = match errno with
         `Ignore_errno -> ()
@@ -442,43 +450,12 @@ struct
       fprintf fmt "}@\n";
     end
 
-  let rec camlxParam fmt args =
-    match args with
-      [] -> ()
-    | x1 :: [] ->
-      fprintf fmt "@[CAMLxparam1 (%s)@];@\n" x1
-    | x1 :: x2 :: [] ->
-      fprintf fmt "@[CAMLxparam2 (%s, %s)@];@\n" x1 x2
-    | x1 :: x2 :: x3 :: [] ->
-      fprintf fmt "@[CAMLxparam3 (%s, %s, %s)@];@\n" x1 x2 x3
-    | x1 :: x2 :: x3 :: x4 :: [] ->
-      fprintf fmt "@[CAMLxparam4 (%s, %s, %s, %s)@];@\n" x1 x2 x3 x4
-    | x1 :: x2 :: x3 :: x4 :: x5 :: rest ->
-      fprintf fmt "@[CAMLxparam5 (%s, %s, %s, %s, %s)@];@\n" x1 x2 x3 x4 x5;
-      camlxParam fmt rest
-
-  let camlParam fmt args =
-    match args with
-      [] ->
-      fprintf fmt "@[CAMLparam0 ()@];@\n"
-    | x1 :: [] ->
-      fprintf fmt "@[CAMLparam1 (%s)@];@\n" x1
-    | x1 :: x2 :: [] ->
-      fprintf fmt "@[CAMLparam2 (%s, %s)@];@\n" x1 x2
-    | x1 :: x2 :: x3 :: [] ->
-      fprintf fmt "@[CAMLparam3 (%s, %s, %s)@];@\n" x1 x2 x3
-    | x1 :: x2 :: x3 :: x4 :: [] ->
-      fprintf fmt "@[CAMLparam4 (%s, %s, %s, %s)@];@\n" x1 x2 x3 x4
-    | x1 :: x2 :: x3 :: x4 :: x5 :: rest ->
-      fprintf fmt "@[CAMLparam5 (%s, %s, %s, %s, %s)@];@\n" x1 x2 x3 x4 x5;
-      camlxParam fmt rest
-
   let stub ~errno ~stub_name fmt fn args =
     begin
       fprintf fmt "@[value@ %s@;@[(%s)@]@]@;@[<2>{@\n"
         stub_name
         (String.concat ", " (List.map (fun (_, x) -> "value "^ x) args));
-      camlParam fmt (List.map snd args);
+      Cstubs_emit_c.camlParam fmt (List.map snd args);
 
       fprintf fmt "@[LWT_UNIX_INIT_JOB(job,@ %s,@ 0)@];@\n"
         stub_name;
