@@ -2,6 +2,7 @@ package widgets
 
 import (
 	"fmt"
+	"strings"
 
 	"github.com/gdamore/tcell/v2"
 	"github.com/mattn/go-runewidth"
@@ -11,7 +12,6 @@ import (
 )
 
 type Selector struct {
-	ui.Invalidatable
 	chooser  bool
 	focused  bool
 	focus    int
@@ -36,7 +36,7 @@ func (sel *Selector) Chooser(chooser bool) *Selector {
 }
 
 func (sel *Selector) Invalidate() {
-	sel.DoInvalidate(sel)
+	ui.Invalidate()
 }
 
 func (sel *Selector) Draw(ctx *ui.Context) {
@@ -166,7 +166,6 @@ func (sel *Selector) Event(event tcell.Event) bool {
 var ErrNoOptionSelected = fmt.Errorf("no option selected")
 
 type SelectorDialog struct {
-	ui.Invalidatable
 	callback func(string, error)
 	title    string
 	prompt   string
@@ -180,13 +179,10 @@ func NewSelectorDialog(title string, prompt string, options []string, focus int,
 	sd := &SelectorDialog{
 		callback: cb,
 		title:    title,
-		prompt:   prompt,
+		prompt:   strings.TrimSpace(prompt),
 		uiConfig: uiConfig,
 		selector: NewSelector(options, focus, uiConfig).Chooser(true),
 	}
-	sd.selector.OnInvalidate(func(_ ui.Drawable) {
-		sd.Invalidate()
-	})
 	sd.selector.Focus(true)
 	return sd
 }
@@ -198,12 +194,37 @@ func (gp *SelectorDialog) Draw(ctx *ui.Context) {
 	ctx.Fill(0, 0, ctx.Width(), ctx.Height(), ' ', defaultStyle)
 	ctx.Fill(0, 0, ctx.Width(), 1, ' ', titleStyle)
 	ctx.Printf(1, 0, titleStyle, "%s", gp.title)
-	ctx.Printf(1, 1, defaultStyle, gp.prompt)
-	gp.selector.Draw(ctx.Subcontext(1, 3, ctx.Width()-2, 1))
+	var i int
+	lines := strings.Split(gp.prompt, "\n")
+	for i = 0; i < len(lines); i++ {
+		ctx.Printf(1, 2+i, defaultStyle, "%s", lines[i])
+	}
+	gp.selector.Draw(ctx.Subcontext(1, ctx.Height()-1, ctx.Width()-2, 1))
+}
+
+func (gp *SelectorDialog) ContextHeight() (func(int) int, func(int) int) {
+	totalHeight := 2 // title + empty line
+	totalHeight += strings.Count(gp.prompt, "\n") + 1
+	totalHeight += 2 // empty line + selector
+	start := func(h int) int {
+		s := h/2 - totalHeight/2
+		if s < 0 {
+			s = 0
+		}
+		return s
+	}
+	height := func(h int) int {
+		if totalHeight > h {
+			return h
+		} else {
+			return totalHeight
+		}
+	}
+	return start, height
 }
 
 func (gp *SelectorDialog) Invalidate() {
-	gp.DoInvalidate(gp)
+	ui.Invalidate()
 }
 
 func (gp *SelectorDialog) Event(event tcell.Event) bool {
