@@ -1,18 +1,42 @@
 package lib
 
 import (
-	"bytes"
 	"io"
-	"io/ioutil"
+	"os"
 	"path/filepath"
 	"testing"
 
 	"git.sr.ht/~rjarry/aerc/models"
 )
 
+func TestMessageInfoParser(t *testing.T) {
+	rootDir := "testdata/message/valid"
+	msgFiles, err := os.ReadDir(rootDir)
+	die(err)
+
+	for _, fi := range msgFiles {
+		if fi.IsDir() {
+			continue
+		}
+
+		p := fi.Name()
+		t.Run(p, func(t *testing.T) {
+			m := newMockRawMessageFromPath(filepath.Join(rootDir, p))
+			mi, err := MessageInfo(m)
+			if err != nil {
+				t.Fatal("Failed to create MessageInfo with:", err)
+			}
+
+			if perr := mi.Error; perr != nil {
+				t.Fatal("Expected no parsing error, but got:", mi.Error)
+			}
+		})
+	}
+}
+
 func TestMessageInfoHandledError(t *testing.T) {
 	rootDir := "testdata/message/invalid"
-	msgFiles, err := ioutil.ReadDir(rootDir)
+	msgFiles, err := os.ReadDir(rootDir)
 	die(err)
 
 	for _, fi := range msgFiles {
@@ -36,27 +60,21 @@ func TestMessageInfoHandledError(t *testing.T) {
 }
 
 type mockRawMessage struct {
-	body []byte
-}
-
-func newMockRawMessage(body []byte) *mockRawMessage {
-	return &mockRawMessage{
-		body: body,
-	}
+	path string
 }
 
 func newMockRawMessageFromPath(p string) *mockRawMessage {
-	b, err := ioutil.ReadFile(p)
-	die(err)
-	return newMockRawMessage(b)
+	return &mockRawMessage{
+		path: p,
+	}
 }
 
-func (m *mockRawMessage) NewReader() (io.Reader, error) {
-	return bytes.NewReader(m.body), nil
+func (m *mockRawMessage) NewReader() (io.ReadCloser, error) {
+	return os.Open(m.path)
 }
-func (m *mockRawMessage) ModelFlags() ([]models.Flag, error) { return nil, nil }
-func (m *mockRawMessage) Labels() ([]string, error)          { return nil, nil }
-func (m *mockRawMessage) UID() uint32                        { return 0 }
+func (m *mockRawMessage) ModelFlags() (models.Flags, error) { return 0, nil }
+func (m *mockRawMessage) Labels() ([]string, error)         { return nil, nil }
+func (m *mockRawMessage) UID() uint32                       { return 0 }
 
 func die(err error) {
 	if err != nil {
