@@ -13,12 +13,14 @@ import (
 	"net/url"
 	"os"
 	"os/exec"
+	"path"
 	"path/filepath"
+	"strconv"
 	"strings"
 	"time"
 
 	"git.sr.ht/~rjarry/aerc/config"
-	"git.sr.ht/~rjarry/aerc/logging"
+	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/handlers"
 	"git.sr.ht/~rjarry/aerc/worker/lib"
@@ -33,8 +35,6 @@ func init() {
 
 var errUnsupported = fmt.Errorf("unsupported command")
 
-const backgroundRefreshDelay = 1 * time.Minute
-
 type worker struct {
 	w                   *types.Worker
 	nmEvents            chan eventType
@@ -42,17 +42,30 @@ type worker struct {
 	currentQueryName    string
 	queryMapOrder       []string
 	nameQueryMap        map[string]string
+	store               *lib.MaildirStore
 	db                  *notmuch.DB
 	setupErr            error
 	currentSortCriteria []*types.SortCriterion
+	watcher             types.FSWatcher
+	watcherDebounce     *time.Timer
+	capabilities        *models.Capabilities
 }
 
 // NewWorker creates a new notmuch worker with the provided worker.
 func NewWorker(w *types.Worker) (types.Backend, error) {
 	events := make(chan eventType, 20)
+	watcher, err := handlers.NewWatcher()
+	if err != nil {
+		return nil, fmt.Errorf("(%s) could not create file system watcher: %w", w.Name, err)
+	}
 	return &worker{
 		w:        w,
 		nmEvents: events,
+		watcher:  watcher,
+		capabilities: &models.Capabilities{
+			Sort:   true,
+			Thread: true,
+		},
 	}, nil
 }
 
@@ -66,23 +79,36 @@ func (w *worker) Run() {
 				w.w.PostMessage(&types.Unsupported{
 					Message: types.RespondTo(msg),
 				}, nil)
-				logging.Errorf("ProcessAction(%T) unsupported: %v", msg, err)
+				log.Errorf("ProcessAction(%T) unsupported: %v", msg, err)
 			} else if err != nil {
 				w.w.PostMessage(&types.Error{
 					Message: types.RespondTo(msg),
 					Error:   err,
 				}, nil)
-				logging.Errorf("ProcessAction(%T) failure: %v", msg, err)
+				log.Errorf("ProcessAction(%T) failure: %v", msg, err)
 			}
 		case nmEvent := <-w.nmEvents:
 			err := w.handleNotmuchEvent(nmEvent)
 			if err != nil {
-				logging.Errorf("notmuch event failure: %v", err)
+				log.Errorf("notmuch event failure: %v", err)
 			}
+		case <-w.watcher.Events():
+			if w.watcherDebounce != nil {
+				w.watcherDebounce.Stop()
+			}
+			// Debounce FS changes
+			w.watcherDebounce = time.AfterFunc(50*time.Millisecond, func() {
+				defer log.PanicHandler()
+				w.nmEvents <- &updateDirCounts{}
+			})
 		}
 	}
 }
 
+func (w *worker) Capabilities() *models.Capabilities {
+	return w.capabilities
+}
+
 func (w *worker) done(msg types.WorkerMessage) {
 	w.w.PostMessage(&types.Done{Message: types.RespondTo(msg)}, nil)
 }
@@ -135,13 +161,18 @@ func (w *worker) handleMessage(msg types.WorkerMessage) error {
 	case *types.CheckMail:
 		go w.handleCheckMail(msg)
 		return nil
-		// not implemented, they are generally not used
-		// in a notmuch based workflow
-		// case *types.DeleteMessages:
-		// case *types.CopyMessages:
-		// case *types.AppendMessage:
-		// case *types.CreateDirectory:
-		// case *types.RemoveDirectory:
+	case *types.DeleteMessages:
+		return w.handleDeleteMessages(msg)
+	case *types.CopyMessages:
+		return w.handleCopyMessages(msg)
+	case *types.MoveMessages:
+		return w.handleMoveMessages(msg)
+	case *types.AppendMessage:
+		return w.handleAppendMessage(msg)
+	case *types.CreateDirectory:
+		return w.handleCreateDirectory(msg)
+	case *types.RemoveDirectory:
+		return w.handleRemoveDirectory(msg)
 	}
 	return errUnsupported
 }
@@ -158,7 +189,7 @@ func (w *worker) handleConfigure(msg *types.Configure) error {
 
 	u, err := url.Parse(msg.Config.Source)
 	if err != nil {
-		logging.Errorf("error configuring notmuch worker: %v", err)
+		log.Errorf("error configuring notmuch worker: %v", err)
 		return err
 	}
 	home, err := homedir.Expand(u.Hostname())
@@ -172,6 +203,20 @@ func (w *worker) handleConfigure(msg *types.Configure) error {
 	}
 	excludedTags := w.loadExcludeTags(msg.Config)
 	w.db = notmuch.NewDB(pathToDB, excludedTags)
+
+	val, ok := msg.Config.Params["maildir-store"]
+	if ok {
+		path, err := homedir.Expand(val)
+		if err != nil {
+			return err
+		}
+		store, err := lib.NewMaildirStore(path, false)
+		if err != nil {
+			return fmt.Errorf("Cannot initialize maildir store: %w", err)
+		}
+		w.store = store
+	}
+
 	return nil
 }
 
@@ -182,102 +227,114 @@ func (w *worker) handleConnect(msg *types.Connect) error {
 	}
 	w.done(msg)
 	w.emitLabelList()
-	go func() {
-		defer logging.PanicHandler()
-
-		for {
-			w.nmEvents <- &updateDirCounts{}
-			time.Sleep(backgroundRefreshDelay)
-		}
-	}()
+	// Watch all the files in the xapian folder for changes. We'll debounce
+	// changes, so catching multiple is ok
+	dbPath := path.Join(w.db.Path(), ".notmuch", "xapian")
+	err = w.watcher.Configure(dbPath)
+	if err != nil {
+		return err
+	}
 	return nil
 }
 
 func (w *worker) handleListDirectories(msg *types.ListDirectories) error {
+	if w.store != nil {
+		folders, err := w.store.FolderMap()
+		if err != nil {
+			log.Errorf("failed listing directories: %v", err)
+			return err
+		}
+		for name := range folders {
+			w.w.PostMessage(&types.Directory{
+				Message: types.RespondTo(msg),
+				Dir: &models.Directory{
+					Name: name,
+				},
+			}, nil)
+		}
+	}
+
 	for _, name := range w.queryMapOrder {
 		w.w.PostMessage(&types.Directory{
 			Message: types.RespondTo(msg),
 			Dir: &models.Directory{
-				Name:       name,
-				Attributes: []string{},
+				Name: name,
+				Role: models.QueryRole,
 			},
 		}, nil)
 	}
+	// Update dir counts when listing directories
+	err := w.handleUpdateDirCounts()
+	if err != nil {
+		return err
+	}
 	w.done(msg)
 	return nil
 }
 
-func (w *worker) gatherDirectoryInfo(name string, query string) (
-	*types.DirectoryInfo, error,
-) {
-	return w.buildDirInfo(name, query, false)
-}
+func (w *worker) getDirectoryInfo(name string, query string) *models.DirectoryInfo {
+	dirInfo := &models.DirectoryInfo{
+		Name: name,
+		// total messages
+		Exists: 0,
+		// new messages since mailbox was last opened
+		Recent: 0,
+		// total unread
+		Unseen: 0,
+	}
 
-func (w *worker) buildDirInfo(name string, query string, skipSort bool) (
-	*types.DirectoryInfo, error,
-) {
 	count, err := w.db.QueryCountMessages(query)
 	if err != nil {
-		return nil, err
+		return dirInfo
 	}
-	info := &types.DirectoryInfo{
-		SkipSort: skipSort,
-		Info: &models.DirectoryInfo{
-			Name:     name,
-			Flags:    []string{},
-			ReadOnly: false,
-			// total messages
-			Exists: count.Exists,
-			// new messages since mailbox was last opened
-			Recent: 0,
-			// total unread
-			Unseen:         count.Unread,
-			AccurateCounts: true,
+	dirInfo.Exists = count.Exists
+	dirInfo.Unseen = count.Unread
 
-			Caps: &models.Capabilities{
-				Sort:   true,
-				Thread: true,
-			},
-		},
-	}
-	return info, nil
-}
-
-func (w *worker) emitDirectoryInfo(name string) error {
-	query, _ := w.queryFromName(name)
-	info, err := w.gatherDirectoryInfo(name, query)
-	if err != nil {
-		return err
-	}
-	w.w.PostMessage(info, nil)
-	return nil
-}
-
-// queryFromName either returns the friendly ID if aliased or the name itself
-// assuming it to be the query
-func (w *worker) queryFromName(name string) (string, bool) {
-	// try the friendly name first, if that fails assume it's a query
-	q, ok := w.nameQueryMap[name]
-	if !ok {
-		return name, true
-	}
-	return q, false
+	return dirInfo
 }
 
 func (w *worker) handleOpenDirectory(msg *types.OpenDirectory) error {
-	logging.Infof("opening %s", msg.Directory)
-	// try the friendly name first, if that fails assume it's a query
-	var isQuery bool
-	w.query, isQuery = w.queryFromName(msg.Directory)
-	w.currentQueryName = msg.Directory
-	info, err := w.gatherDirectoryInfo(msg.Directory, w.query)
-	if err != nil {
-		return err
+	log.Tracef("opening %s", msg.Directory)
+
+	var isDynamicFolder bool
+	q := ""
+	if w.store != nil {
+		folders, _ := w.store.FolderMap()
+		dir, ok := folders[msg.Directory]
+		if ok {
+			q = fmt.Sprintf("folder:%s", strconv.Quote(msg.Directory))
+			if err := w.processNewMaildirFiles(string(dir)); err != nil {
+				return err
+			}
+		}
 	}
-	info.Message = types.RespondTo(msg)
-	w.w.PostMessage(info, nil)
-	if isQuery {
-		w.w.PostMessage(info, nil)
+	if q == "" {
+		var ok bool
+		q, ok = w.nameQueryMap[msg.Directory]
+		if !ok {
+			q = msg.Directory
+			isDynamicFolder = true
+			w.w.PostMessage(&types.Directory{
+				Message: types.RespondTo(msg),
+				Dir: &models.Directory{
+					Name: q,
+					Role: models.QueryRole,
+				},
+			}, nil)
+		}
+	}
+	w.query = q
+	w.currentQueryName = msg.Directory
+
+	w.w.PostMessage(&types.DirectoryInfo{
+		Info:    w.getDirectoryInfo(msg.Directory, w.query),
+		Message: types.RespondTo(msg),
+	}, nil)
+	if isDynamicFolder {
+		w.w.PostMessage(&types.DirectoryInfo{
+			Info:    w.getDirectoryInfo(msg.Directory, w.query),
+			Message: types.RespondTo(msg),
+		}, nil)
 	}
 	w.done(msg)
 	return nil
@@ -313,13 +370,13 @@ func (w *worker) handleFetchMessageHeaders(
 	for _, uid := range msg.Uids {
 		m, err := w.msgFromUid(uid)
 		if err != nil {
-			logging.Errorf("could not get message: %v", err)
+			log.Errorf("could not get message: %v", err)
 			w.w.PostMessageInfoError(msg, uid, err)
 			continue
 		}
 		err = w.emitMessageInfo(m, msg)
 		if err != nil {
-			logging.Errorf("could not emit message info: %v", err)
+			log.Errorf("could not emit message info: %v", err)
 			w.w.PostMessageInfoError(msg, uid, err)
 			continue
 		}
@@ -360,12 +417,12 @@ func (w *worker) handleFetchMessageBodyPart(
 ) error {
 	m, err := w.msgFromUid(msg.Uid)
 	if err != nil {
-		logging.Errorf("could not get message %d: %v", msg.Uid, err)
+		log.Errorf("could not get message %d: %v", msg.Uid, err)
 		return err
 	}
 	r, err := m.NewBodyPartReader(msg.Part)
 	if err != nil {
-		logging.Errorf(
+		log.Errorf(
 			"could not get body part reader for message=%d, parts=%#v: %w",
 			msg.Uid, msg.Part, err)
 		return err
@@ -386,12 +443,12 @@ func (w *worker) handleFetchFullMessages(msg *types.FetchFullMessages) error {
 	for _, uid := range msg.Uids {
 		m, err := w.msgFromUid(uid)
 		if err != nil {
-			logging.Errorf("could not get message %d: %v", uid, err)
+			log.Errorf("could not get message %d: %v", uid, err)
 			return err
 		}
 		r, err := m.NewReader()
 		if err != nil {
-			logging.Errorf("could not get message reader: %v", err)
+			log.Errorf("could not get message reader: %v", err)
 			return err
 		}
 		defer r.Close()
@@ -415,25 +472,25 @@ func (w *worker) handleAnsweredMessages(msg *types.AnsweredMessages) error {
 	for _, uid := range msg.Uids {
 		m, err := w.msgFromUid(uid)
 		if err != nil {
-			logging.Errorf("could not get message: %v", err)
+			log.Errorf("could not get message: %v", err)
 			w.err(msg, err)
 			continue
 		}
 		if err := m.MarkAnswered(msg.Answered); err != nil {
-			logging.Errorf("could not mark message as answered: %v", err)
+			log.Errorf("could not mark message as answered: %v", err)
 			w.err(msg, err)
 			continue
 		}
 		err = w.emitMessageInfo(m, msg)
 		if err != nil {
-			logging.Errorf("could not emit message info: %v", err)
+			log.Errorf("could not emit message info: %v", err)
 			w.err(msg, err)
 			continue
 		}
 	}
-	if err := w.emitDirectoryInfo(w.currentQueryName); err != nil {
-		logging.Errorf("could not emit directory info: %v", err)
-	}
+	w.w.PostMessage(&types.DirectoryInfo{
+		Info: w.getDirectoryInfo(w.currentQueryName, w.query),
+	}, nil)
 	w.done(msg)
 	return nil
 }
@@ -442,25 +499,26 @@ func (w *worker) handleFlagMessages(msg *types.FlagMessages) error {
 	for _, uid := range msg.Uids {
 		m, err := w.msgFromUid(uid)
 		if err != nil {
-			logging.Errorf("could not get message: %v", err)
+			log.Errorf("could not get message: %v", err)
 			w.err(msg, err)
 			continue
 		}
-		if err := m.SetFlag(msg.Flag, msg.Enable); err != nil {
-			logging.Errorf("could not set flag %v as %t for message: %v", msg.Flag, msg.Enable, err)
+		if err := m.SetFlag(msg.Flags, msg.Enable); err != nil {
+			log.Errorf("could not set flag %v as %t for message: %v",
+				msg.Flags, msg.Enable, err)
 			w.err(msg, err)
 			continue
 		}
 		err = w.emitMessageInfo(m, msg)
 		if err != nil {
-			logging.Errorf("could not emit message info: %v", err)
+			log.Errorf("could not emit message info: %v", err)
 			w.err(msg, err)
 			continue
 		}
 	}
-	if err := w.emitDirectoryInfo(w.currentQueryName); err != nil {
-		logging.Errorf("could not emit directory info: %v", err)
-	}
+	w.w.PostMessage(&types.DirectoryInfo{
+		Info: w.getDirectoryInfo(w.currentQueryName, w.query),
+	}, nil)
 	w.done(msg)
 	return nil
 }
@@ -507,9 +565,9 @@ func (w *worker) handleModifyLabels(msg *types.ModifyLabels) error {
 	}
 	// and update the list of possible tags
 	w.emitLabelList()
-	if err = w.emitDirectoryInfo(w.currentQueryName); err != nil {
-		logging.Errorf("could not emit directory info: %v", err)
-	}
+	w.w.PostMessage(&types.DirectoryInfo{
+		Info: w.getDirectoryInfo(w.currentQueryName, w.query),
+	}, nil)
 	w.done(msg)
 	return nil
 }
@@ -541,8 +599,8 @@ func (w *worker) loadQueryMap(acctConfig *config.AccountConfig) error {
 		if len(split) != 2 {
 			return fmt.Errorf("%v: invalid line %q, want name=query", file, line)
 		}
-		w.nameQueryMap[split[0]] = split[1]
-		w.queryMapOrder = append(w.queryMapOrder, split[0])
+		w.nameQueryMap[strings.TrimSpace(split[0])] = split[1]
+		w.queryMapOrder = append(w.queryMapOrder, strings.TrimSpace(split[0]))
 	}
 	return nil
 }
@@ -576,7 +634,7 @@ func (w *worker) emitDirectoryContents(parent types.WorkerMessage) error {
 	}
 	sortedUids, err := w.sort(uids, w.currentSortCriteria)
 	if err != nil {
-		logging.Errorf("error sorting directory: %v", err)
+		log.Errorf("error sorting directory: %v", err)
 		return err
 	}
 	w.w.PostMessage(&types.DirectoryContents{
@@ -621,7 +679,7 @@ func (w *worker) emitMessageInfo(m *Message,
 func (w *worker) emitLabelList() {
 	tags, err := w.db.ListTags()
 	if err != nil {
-		logging.Errorf("could not load tags: %v", err)
+		log.Errorf("could not load tags: %v", err)
 		return
 	}
 	w.w.PostMessage(&types.LabelList{Labels: tags}, nil)
@@ -637,45 +695,271 @@ func (w *worker) sort(uids []uint32,
 	for _, uid := range uids {
 		m, err := w.msgFromUid(uid)
 		if err != nil {
-			logging.Errorf("could not get message: %v", err)
+			log.Errorf("could not get message: %v", err)
 			continue
 		}
 		info, err := m.MessageInfo()
 		if err != nil {
-			logging.Errorf("could not get message info: %v", err)
+			log.Errorf("could not get message info: %v", err)
 			continue
 		}
 		msgInfos = append(msgInfos, info)
 	}
 	sortedUids, err := lib.Sort(msgInfos, criteria)
 	if err != nil {
-		logging.Errorf("could not sort the messages: %v", err)
+		log.Errorf("could not sort the messages: %v", err)
 		return nil, err
 	}
 	return sortedUids, nil
 }
 
 func (w *worker) handleCheckMail(msg *types.CheckMail) {
+	defer log.PanicHandler()
 	if msg.Command == "" {
-		w.err(msg, fmt.Errorf("checkmail: no command specified"))
+		w.err(msg, fmt.Errorf("(%s) checkmail: no command specified", w.w.Name))
 		return
 	}
 	ctx, cancel := context.WithTimeout(context.Background(), msg.Timeout)
 	defer cancel()
 	cmd := exec.CommandContext(ctx, "sh", "-c", msg.Command)
-	ch := make(chan error)
-	go func() {
-		err := cmd.Run()
-		ch <- err
-	}()
-	select {
-	case <-ctx.Done():
-		w.err(msg, fmt.Errorf("checkmail: timed out"))
-	case err := <-ch:
-		if err != nil {
-			w.err(msg, fmt.Errorf("checkmail: error running command: %w", err))
-		} else {
-			w.done(msg)
-		}
+	err := cmd.Run()
+	switch {
+	case ctx.Err() != nil:
+		w.err(msg, fmt.Errorf("(%s) checkmail: timed out", w.w.Name))
+	case err != nil:
+		w.err(msg, fmt.Errorf("(%s) checkmail: error running command: %w", w.w.Name, err))
+	default:
+		w.done(msg)
 	}
 }
+
+func (w *worker) handleDeleteMessages(msg *types.DeleteMessages) error {
+	if w.store == nil {
+		return errUnsupported
+	}
+
+	var deleted []uint32
+
+	// With notmuch, two identical files can be referenced under
+	// the same index key, even if they exist in two different
+	// folders. So in order to remove the message from the right
+	// maildir folder we need to pass a hint to Remove() so it
+	// can purge the right file.
+	folders, _ := w.store.FolderMap()
+	path, ok := folders[w.currentQueryName]
+	if !ok {
+		w.err(msg, fmt.Errorf("Can only delete file from a maildir folder"))
+		w.done(msg)
+		return nil
+	}
+
+	for _, uid := range msg.Uids {
+		m, err := w.msgFromUid(uid)
+		if err != nil {
+			log.Errorf("could not get message: %v", err)
+			w.err(msg, err)
+			continue
+		}
+		if err := m.Remove(path); err != nil {
+			log.Errorf("could not remove message: %v", err)
+			w.err(msg, err)
+			continue
+		}
+		deleted = append(deleted, uid)
+	}
+	if len(deleted) > 0 {
+		w.w.PostMessage(&types.MessagesDeleted{
+			Message: types.RespondTo(msg),
+			Uids:    deleted,
+		}, nil)
+	}
+	w.done(msg)
+	return nil
+}
+
+func (w *worker) handleCopyMessages(msg *types.CopyMessages) error {
+	if w.store == nil {
+		return errUnsupported
+	}
+
+	// Only allow file to be copied to a maildir folder
+	folders, _ := w.store.FolderMap()
+	dest, ok := folders[msg.Destination]
+	if !ok {
+		return fmt.Errorf("Can only copy file to a maildir folder")
+	}
+
+	for _, uid := range msg.Uids {
+		m, err := w.msgFromUid(uid)
+		if err != nil {
+			log.Errorf("could not get message: %v", err)
+			return err
+		}
+		if err := m.Copy(dest); err != nil {
+			log.Errorf("could not copy message: %v", err)
+			return err
+		}
+	}
+	w.w.PostMessage(&types.MessagesCopied{
+		Message:     types.RespondTo(msg),
+		Destination: msg.Destination,
+		Uids:        msg.Uids,
+	}, nil)
+	w.done(msg)
+	return nil
+}
+
+func (w *worker) handleMoveMessages(msg *types.MoveMessages) error {
+	if w.store == nil {
+		return errUnsupported
+	}
+
+	var moved []uint32
+
+	folders, _ := w.store.FolderMap()
+
+	// Only allow file to be moved to a maildir folder
+	dest, ok := folders[msg.Destination]
+	if !ok {
+		return fmt.Errorf("Can only move file to a maildir folder")
+	}
+
+	var err error
+	for _, uid := range msg.Uids {
+		m, err := w.msgFromUid(uid)
+		if err != nil {
+			log.Errorf("could not get message: %v", err)
+			break
+		}
+		filenames, err := m.db.MsgFilenames(m.key)
+		if err != nil {
+			return err
+		}
+		// In the future, it'd be nice if we could overload move with
+		// the possibility to affect some or all of the files
+		// corresponding to a message.
+		if len(filenames) > 1 {
+			return fmt.Errorf("Cannot move: message %d has multiple files", m.uid)
+		}
+		source, key := parseFilename(filenames[0])
+		if key == "" {
+			return fmt.Errorf("failed to parse message filename: %s", filenames[0])
+		}
+		if err := m.Move(source, dest); err != nil {
+			log.Errorf("could not copy message: %v", err)
+			break
+		}
+		moved = append(moved, uid)
+	}
+	w.w.PostMessage(&types.MessagesDeleted{
+		Message: types.RespondTo(msg),
+		Uids:    moved,
+	}, nil)
+	if err == nil {
+		w.done(msg)
+	}
+	return err
+}
+
+func (w *worker) handleAppendMessage(msg *types.AppendMessage) error {
+	if w.store == nil {
+		return errUnsupported
+	}
+
+	// Only allow file to be created in a maildir folder
+	// since we are the "master" maildir process, we can modify the maildir directly
+	folders, _ := w.store.FolderMap()
+	dest, ok := folders[msg.Destination]
+	if !ok {
+		return fmt.Errorf("Can only create file in a maildir folder")
+	}
+	key, writer, err := dest.Create(lib.ToMaildirFlags(msg.Flags))
+	if err != nil {
+		log.Errorf("could not create message at %s: %v", msg.Destination, err)
+		return err
+	}
+	filename, err := dest.Filename(key)
+	if err != nil {
+		writer.Close()
+		return err
+	}
+	if _, err := io.Copy(writer, msg.Reader); err != nil {
+		log.Errorf("could not write message to destination: %v", err)
+		writer.Close()
+		os.Remove(filename)
+		return err
+	}
+	writer.Close()
+	if _, err := w.db.IndexFile(filename); err != nil {
+		return err
+	}
+	w.w.PostMessage(&types.DirectoryInfo{
+		Info: w.getDirectoryInfo(w.currentQueryName, w.query),
+	}, nil)
+	w.done(msg)
+	return nil
+}
+
+func (w *worker) handleCreateDirectory(msg *types.CreateDirectory) error {
+	if w.store == nil {
+		return errUnsupported
+	}
+
+	dir := w.store.Dir(msg.Directory)
+	if err := dir.Init(); err != nil {
+		log.Errorf("could not create directory %s: %v",
+			msg.Directory, err)
+		return err
+	}
+	w.done(msg)
+	return nil
+}
+
+func (w *worker) handleRemoveDirectory(msg *types.RemoveDirectory) error {
+	if w.store == nil {
+		return errUnsupported
+	}
+
+	dir := w.store.Dir(msg.Directory)
+	if err := os.RemoveAll(string(dir)); err != nil {
+		log.Errorf("could not remove directory %s: %v",
+			msg.Directory, err)
+		return err
+	}
+	w.done(msg)
+	return nil
+}
+
+// This is a hack that calls MsgModifyTags with an empty list of tags to
+// apply on new messages causing notmuch to rename files and effectively
+// move them into the cur/ dir.
+func (w *worker) processNewMaildirFiles(dir string) error {
+	f, err := os.Open(filepath.Join(dir, "new"))
+	if err != nil {
+		return err
+	}
+	defer f.Close()
+	names, err := f.Readdirnames(0)
+	if err != nil {
+		return err
+	}
+
+	for _, n := range names {
+		if n[0] == '.' {
+			continue
+		}
+
+		key, err := w.db.MsgIDFromFilename(filepath.Join(dir, "new", n))
+		if err != nil {
+			// Message is not yet indexed, leave it alone
+			continue
+		}
+		// Force message to move from new/ to cur/
+		err = w.db.MsgModifyTags(key, nil, nil)
+		if err != nil {
+			log.Errorf("MsgModifyTags failed: %v", err)
+		}
+	}
+
+	return nil
+}
